import { __decorate, __metadata } from 'tslib';
import { Prop, Component } from 'vue-property-decorator';
import { classNamesFunction } from '@uifabric-vue/utilities';
import BaseComponent from '../BaseComponent.js';
import { Icon } from '../Icon/index.js';
import { ActionButton } from '../Button/ActionButton/ActionButton.js';

const getClassNames = classNamesFunction(); // The number pixels per indentation level for Nav links.

const INDENTATION_SIZE = 14; // The number of pixels of left margin

const BASE_INDENT = 3;
let Nav = class Nav extends BaseComponent {
  constructor() {
    super(...arguments);
    this.isGroupCollapsed = {};
    this.internalSelectedKey = '';
  }

  render(h) {
    const {
      theme,
      className,
      isOnTop,
      groups
    } = this;

    if (!groups) {
      return null;
    }

    const groupElements = this.groups.map(this.renderGroup);
    const classNames = getClassNames(this.styles, {
      theme: theme,
      className,
      isOnTop,
      groups
    });
    return h("nav", {
      "attrs": {
        "role": "navigation"
      },
      "class": classNames.root
    }, [groupElements]);
  }

  renderGroup(group, groupIndex) {
    const h = this.$createElement;
    const {
      groups,
      theme
    } = this;
    const classNames = getClassNames(this.styles, {
      theme: theme,
      isGroup: true,
      isExpanded: this.isGroupExpanded(group),
      groups
    });
    return h("div", {
      "key": groupIndex
    }, [h("div", {
      "class": classNames.groupContent
    }, [this.renderLinks(group.links, 0)])]);
  }

  renderLinks(links, nestingLevel) {
    const h = this.$createElement;
    if (!links || !links.length) return null;
    const linkElements = links.map((link, linkIndex) => this.renderLink(link, linkIndex, nestingLevel));
    const {
      groups,
      theme
    } = this;
    const classNames = getClassNames(this.styles, {
      theme: theme,
      groups
    });
    return h("ul", {
      "attrs": {
        "role": "list"
      },
      "class": classNames.navItems
    }, [linkElements]);
  }

  renderLink(link, linkIndex, nestingLevel) {
    const h = this.$createElement;
    const {
      groups,
      theme
    } = this;
    const classNames = getClassNames(this.styles, {
      theme: theme,
      groups
    });
    return h("li", {
      "key": linkIndex,
      "attrs": {
        "role": "listitem"
      },
      "class": classNames.navItem
    }, [this.renderCompositeLink(link, linkIndex, nestingLevel), link.isExpanded ? this.renderLinks(link.links, ++nestingLevel) : null]);
  }

  renderCompositeLink(link, linkIndex, nestingLevel) {
    const h = this.$createElement;
    const {
      styles,
      groups,
      theme
    } = this;
    const classNames = getClassNames(this.styles, {
      theme: theme,
      isExpanded: !!link.isExpanded,
      isSelected: false,
      isLink: true,
      isDisabled: link.disabled,
      position: INDENTATION_SIZE * nestingLevel + 1,
      groups
    });
    return h("div", {
      "key": linkIndex,
      "class": classNames.compositeLink
    }, [link.links && link.links.length > 0 ? h("button", {
      "on": {
        "click": this.onLinkExpandClicked.bind(this, link)
      },
      "class": classNames.chevronButton
    }, [h(Icon, {
      "attrs": {
        "class-name": classNames.chevronIcon,
        "icon-name": "ChevronDown"
      },
      "style": link.isExpanded && {
        transform: 'rotate(-180deg)'
      }
    })]) : null, this.renderNavLink(link, linkIndex, nestingLevel)]);
  }

  onLinkExpandClicked(link) {
    link.isExpanded = !link.isExpanded;
  }

  renderNavLink(link, linkIndex, nestingLevel) {
    const h = this.$createElement;
    const isSelected = link.key === this.internalSelectedKey;
    const isLinkWithIcon = link.icon || link.iconProps;
    const {
      groups,
      theme
    } = this;
    const classNames = getClassNames(this.styles, {
      theme: theme,
      isSelected: isSelected,
      isDisabled: link.disabled,
      isButtonEntry: link.onClick && !link.forceAnchor,
      leftPadding: INDENTATION_SIZE * nestingLevel + BASE_INDENT + (isLinkWithIcon ? 0 : 24),
      groups
    });
    return h(ActionButton, {
      "attrs": {
        "href": link.href || (link.forceAnchor ? '#' : undefined),
        "title": link.title || link.name,
        "target": link.target,
        "disabled": link.disabled
      },
      "nativeOn": {
        "click": this.onNavLinkClicked.bind(this, link, linkIndex)
      },
      "class": [classNames.link, isSelected && classNames.selected],
      "style": {
        paddingLeft: `${INDENTATION_SIZE * nestingLevel + BASE_INDENT + (isLinkWithIcon ? 0 : 24)}px`
      }
    }, [link.name]);
  }

  onNavLinkClicked(link, linkIndex) {
    if (link.onClick) link.onClick(link);

    if (!link.url && link.links && link.links.length > 0) {
      link.isExpanded = !link.isExpanded;
    }

    this.internalSelectedKey = link.key;
  }

  preventBounce(link, ev) {
    if (!link.href && link.forceAnchor) ev.preventDefault();
  }

  isGroupExpanded(group) {
    if (group.name && this.isGroupCollapsed.hasOwnProperty(group.name)) {
      return !this.isGroupCollapsed[group.name];
    }

    if (group.collapseByDefault !== undefined) {
      return !group.collapseByDefault;
    }

    return true;
  }

  toggleCollapsed(group) {
    if (group.name) {
      const newGroupCollapsed = Object.assign(Object.assign({}, this.isGroupCollapsed), {
        [group.name]: this.isGroupExpanded(group)
      });
      this.$set(this, 'isGroupCollapsed', newGroupCollapsed);
    }
  }

};

__decorate([Prop({
  type: Array,
  default: () => []
}), __metadata("design:type", Array)], Nav.prototype, "groups", void 0);

__decorate([Prop({
  type: String,
  default: ''
}), __metadata("design:type", String)], Nav.prototype, "selectedKey", void 0);

__decorate([Prop({
  type: Boolean,
  default: false
}), __metadata("design:type", Boolean)], Nav.prototype, "isOnTop", void 0);

Nav = __decorate([Component({
  components: {
    ActionButton,
    Icon
  }
})], Nav);
var NavBase = Nav;

export default NavBase;
